<?php
/*
Plugin Name: Modo Mantenimiento
Plugin URI: https://felipejurado.com/plugin-wordpress-modo-mantenimiento
Description: Pone la web en modo mantenimiento, activándose y desactivándose en la barra de usuario para administradores.
Version: 1.5.2
Author: Felipe Jurado
Author URI: https://www.felipejurado.com
License: GPL v2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Requires at least: 5.0
Requires PHP: 7.2
*/

// Función para mostrar la página de mantenimiento
function modo_mantenimiento() {
    if (!current_user_can('manage_options') && !is_user_logged_in()) {
        // Añadir cabeceras para prevenir el almacenamiento en caché
        nocache_headers();
        header("HTTP/1.1 503 Service Temporarily Unavailable");
        header("Status: 503 Service Temporarily Unavailable");
        header("Retry-After: 3600");
        
        // Obtener opciones
        $mostrar_texto = get_option('mantenimiento_mostrar_texto', 'no');
        $mostrar_imagen = get_option('mantenimiento_mostrar_imagen', 'si');
        $texto_mantenimiento = get_option('mantenimiento_texto', 'Estamos de mantenimiento, volvemos lo antes posible.');
        $imagen_mantenimiento = get_option('mantenimiento_imagen', plugin_dir_url(__FILE__) . 'img/mantenimiento.png');
        
        // Estilos CSS
        $estilos = "
            html, body {
                margin: 0;
                padding: 0;
                width: 100%;
                height: 100%;
                display: flex;
                flex-direction: column;
                justify-content: center;
                align-items: center;
                background: #f0f0f0;
                text-align: center;
                font-family: Arial, sans-serif;
            }
            img {
                max-width: 100%;
                max-height: 70%;
                height: auto;
                width: auto;
            }
            p {
                margin-top: 20px;
                font-size: 24px;
                color: #333;
            }
        ";

        // Contenido HTML
        $html = '<!DOCTYPE html><html><head>';
        $html .= '<title>' . esc_html__('Sitio en mantenimiento', 'modo-mantenimiento') . '</title>';
        $html .= '<meta name="robots" content="noindex,nofollow">';
        $html .= '<style>' . esc_html($estilos) . '</style>';
        $html .= '</head><body>';

        if ($mostrar_imagen === 'si' && $imagen_mantenimiento) {
            $html .= '<img src="' . esc_url($imagen_mantenimiento) . '" alt="Mantenimiento">';
        }
        
        if ($mostrar_texto === 'si') {
            $html .= '<p>' . esc_html($texto_mantenimiento) . '</p>';
        }

        $html .= '</body></html>';
        
        echo $html;
        exit;
    }
}

// Añadir la opción en la barra de administración
add_action('admin_bar_menu', 'mantenimiento_admin_bar_menu', 100);
function mantenimiento_admin_bar_menu($wp_admin_bar) {
    if (current_user_can('manage_options')) {
        $estado = get_option('modo_mantenimiento_activado', 'no');
        $icon_url = plugin_dir_url( __FILE__ ) . 'img/' . ($estado == 'si' ? 'led_rojo.png' : 'led_verde.png');
        $title = '| <img src="' . esc_url($icon_url) . '" style="vertical-align: middle; margin-right: 5px;">' . ($estado == 'si' ? 'Web en Mantenimiento' : 'Web Activa') . ' |';
        $url = wp_nonce_url(admin_url('admin-ajax.php?action=toggle_mantenimiento'), 'toggle-mantenimiento');
        $wp_admin_bar->add_menu(array(
            'id'    => 'modo_mantenimiento',
            'title' => $title,
            'href'  => $url
        ));
    }
}

// Manejar la activación/desactivación
add_action('wp_ajax_toggle_mantenimiento', 'toggle_mantenimiento');
function toggle_mantenimiento() {
    if (!current_user_can('manage_options')) {
        wp_die('No tienes permisos para hacer esto');
    }

    check_admin_referer('toggle-mantenimiento');

    $estado = get_option('modo_mantenimiento_activado', 'no');
    $nuevo_estado = ($estado == 'no') ? 'si' : 'no';
    update_option('modo_mantenimiento_activado', $nuevo_estado);

    clear_all_caches();

    wp_redirect(admin_url());
    exit;
}

// Activar/desactivar el modo mantenimiento
add_action('template_redirect', 'check_modo_mantenimiento', 1);
function check_modo_mantenimiento() {
    if (get_option('modo_mantenimiento_activado', 'no') == 'si') {
        modo_mantenimiento();
    }
}

// Añadir submenú en "Ajustes"
add_action('admin_menu', 'mantenimiento_admin_menu');
function mantenimiento_admin_menu() {
    add_options_page(
        'Configuración de Mantenimiento',
        'Mantenimiento',
        'manage_options',
        'mantenimiento-configuracion',
        'mantenimiento_configuracion_page'
    );
}

// Contenido de la página de configuración
function mantenimiento_configuracion_page() {
    ?>
    <div class="wrap">
        <h1>Configuración de Mantenimiento</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('mantenimiento_configuracion');
            do_settings_sections('mantenimiento-configuracion');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Registrar configuraciones
add_action('admin_init', 'mantenimiento_registrar_configuraciones');
function mantenimiento_registrar_configuraciones() {
    register_setting('mantenimiento_configuracion', 'mantenimiento_mostrar_texto');
    register_setting('mantenimiento_configuracion', 'mantenimiento_texto');
    register_setting('mantenimiento_configuracion', 'mantenimiento_mostrar_imagen');
    register_setting('mantenimiento_configuracion', 'mantenimiento_imagen');

    add_settings_section(
        'mantenimiento_configuracion_seccion',
        'Opciones de Mantenimiento',
        null,
        'mantenimiento-configuracion'
    );

    add_settings_field('mantenimiento_mostrar_texto', 'Mostrar texto', 'mantenimiento_mostrar_texto_callback', 'mantenimiento-configuracion', 'mantenimiento_configuracion_seccion');
    add_settings_field('mantenimiento_texto', 'Texto de Mantenimiento', 'mantenimiento_texto_callback', 'mantenimiento-configuracion', 'mantenimiento_configuracion_seccion');
    add_settings_field('mantenimiento_mostrar_imagen', 'Mostrar imagen', 'mantenimiento_mostrar_imagen_callback', 'mantenimiento-configuracion', 'mantenimiento_configuracion_seccion');
    add_settings_field('mantenimiento_imagen', 'Imagen de Mantenimiento', 'mantenimiento_imagen_callback', 'mantenimiento-configuracion', 'mantenimiento_configuracion_seccion');
}

function mantenimiento_admin_enqueue_scripts($hook) {
    if ('settings_page_mantenimiento-configuracion' !== $hook) {
        return;
    }
    wp_enqueue_media();
    wp_enqueue_script('mantenimiento-media-script', plugin_dir_url(__FILE__) . 'js/mantenimiento-media.js', array('jquery'), '1.0', true);
}
add_action('admin_enqueue_scripts', 'mantenimiento_admin_enqueue_scripts');

function mantenimiento_mostrar_texto_callback() {
    $opcion = get_option('mantenimiento_mostrar_texto', 'no');
    ?>
    <input type="radio" name="mantenimiento_mostrar_texto" value="si" <?php checked($opcion, 'si'); ?>> Sí
    <input type="radio" name="mantenimiento_mostrar_texto" value="no" <?php checked($opcion, 'no'); ?>> No
    <?php
}

function mantenimiento_texto_callback() {
    $texto = get_option('mantenimiento_texto', 'Estamos de mantenimiento, volvemos lo antes posible.');
    ?>
    <textarea name="mantenimiento_texto" rows="4" cols="50"><?php echo esc_textarea($texto); ?></textarea>
    <?php
}

function mantenimiento_mostrar_imagen_callback() {
    $opcion = get_option('mantenimiento_mostrar_imagen', 'si');
    ?>
    <input type="radio" name="mantenimiento_mostrar_imagen" value="si" <?php checked($opcion, 'si'); ?>> Sí
    <input type="radio" name="mantenimiento_mostrar_imagen" value="no" <?php checked($opcion, 'no'); ?>> No
    <?php
}

function mantenimiento_imagen_callback() {
    $imagen = get_option('mantenimiento_imagen', plugin_dir_url( __FILE__ ) . 'img/mantenimiento.png');
    ?>
    <input type="text" name="mantenimiento_imagen" id="mantenimiento_imagen" value="<?php echo esc_url($imagen); ?>" style="width: 70%;">
    <input type="button" id="mantenimiento_imagen_button" class="button" value="Seleccionar imagen">
    <?php if ($imagen) : ?>
        <img src="<?php echo esc_url($imagen); ?>" style="max-width:200px; height:auto; margin-top:10px;" />
    <?php endif;
}

function clear_all_caches() {
    if (function_exists('wp_cache_flush')) {
        wp_cache_flush();
    }
    if (function_exists('wp_cache_clear_cache')) {
        wp_cache_clear_cache();
    }
    if (function_exists('w3tc_flush_all')) {
        w3tc_flush_all();
    }
    if (function_exists('wp_fastest_cache_clear_all_cache')) {
        wp_fastest_cache_clear_all_cache();
    }
    if (function_exists('rocket_clean_domain')) {
        rocket_clean_domain();
    }
    nocache_headers();
}